"use strict";

var _require = require('@babel/helper-plugin-utils'),
    declare = _require.declare;

var syntaxJsx = require('@babel/plugin-syntax-jsx').default;

var path = require('path');

var _require2 = require('./utils'),
    fileExists = _require2.fileExists,
    isFunction = _require2.isFunction;

var options = require('./options');

var transformClass = require('./plugins/transform-class');

var injectScope = require('./plugins/inject-scope');

var pkg;

if (fileExists(path.join(process.cwd(), 'package.json'))) {
  pkg = require(path.join(process.cwd(), 'package.json'));
}

function pluginHook(plugin) {
  if (!isFunction(plugin)) return plugin;
  return function () {
    var ret = plugin.apply(this, arguments);
    if (!ret.inherits) ret.inherits = syntaxJsx;
    return ret;
  };
}

var loaded = false;
module.exports = declare(function (api) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  api.assertVersion(7);
  if (!opts) opts = {};

  if (!loaded) {
    loaded = true;
    Object.assign(options, opts);
    options.pkg = pkg;
  }

  var plugins = [];
  if (options.class) plugins.push(transformClass);
  plugins.push(injectScope);
  plugins = plugins.map(function (p) {
    return pluginHook(p);
  });
  return {
    plugins: plugins
  };
});